package org.apache.bcel.generic;

import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.util.ByteSequence;
import org.apache.bcel.ExceptionConst;
import java.io.IOException;
import java.io.DataOutputStream;

public final class INVOKEINTERFACE extends InvokeInstruction
{
    private int nargs;
    
    INVOKEINTERFACE() {
    }
    
    public INVOKEINTERFACE(final int index, final int nargs) {
        super((short)185, index);
        super.setLength(5);
        if (nargs < 1) {
            throw new ClassGenException("Number of arguments must be > 0 " + nargs);
        }
        this.nargs = nargs;
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitExceptionThrower(this);
        v.visitTypedInstruction(this);
        v.visitStackConsumer(this);
        v.visitStackProducer(this);
        v.visitLoadClass(this);
        v.visitCPInstruction(this);
        v.visitFieldOrMethod(this);
        v.visitInvokeInstruction(this);
        v.visitINVOKEINTERFACE(this);
    }
    
    @Override
    public int consumeStack(final ConstantPoolGen cpg) {
        return this.nargs;
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        out.writeByte(super.getOpcode());
        out.writeShort(super.getIndex());
        out.writeByte(this.nargs);
        out.writeByte(0);
    }
    
    public int getCount() {
        return this.nargs;
    }
    
    @Override
    public Class<?>[] getExceptions() {
        return ExceptionConst.createExceptions(ExceptionConst.EXCS.EXCS_INTERFACE_METHOD_RESOLUTION, ExceptionConst.UNSATISFIED_LINK_ERROR, ExceptionConst.ABSTRACT_METHOD_ERROR, ExceptionConst.ILLEGAL_ACCESS_ERROR, ExceptionConst.INCOMPATIBLE_CLASS_CHANGE_ERROR);
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        super.initFromFile(bytes, wide);
        super.setLength(5);
        this.nargs = bytes.readUnsignedByte();
        bytes.readByte();
    }
    
    @Override
    public String toString(final ConstantPool cp) {
        return super.toString(cp) + " " + this.nargs;
    }
}
